/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <media_content.h>
#include <app_debug.h>
#include <app_media.h>

#include "data/mediadata.h"

typedef int (*media_compare)(struct group_info *gi, app_media_info *info);
typedef void *(*media_compare_data_get)(app_media_info *info);
typedef char *(*group_name_get)(app_media_info *info);

static int _compare_cb_date(const void *, const void *);
static int _compare_cb_name(const void *, const void *);

static int _compare_title(struct group_info *, app_media_info *);
static char *_get_title(app_media_info *);
static void *_get_data_title(app_media_info *info);
static int _compare_time(struct group_info *, app_media_info *);
static char *_get_time(app_media_info *);
static void *_get_data_time(app_media_info *info);

enum _filter_type {
	E_FILTER_FOLDER = 0,
	E_FILTER_MEDIA
};

struct mediadata {
	Eina_List *media_list;

	int media_count[E_MEDIA_MAX];
};

struct _data {
	const char *media_type;
	int source_type;
	int sort_type;
	Eina_List *media_list;
};

struct _sort_info {
	int order_type;
	const char *order_keyword;
	Eina_Compare_Cb func;
};

struct _list_info {
	media_compare media_cmp;
	media_compare_data_get data_get;
	group_name_get name_get;
};

static struct _sort_info g_sort_info[E_SORT_MAX] = {
	[E_SORT_DATE] = {
		.order_type = MEDIA_CONTENT_ORDER_DESC,
		.order_keyword = MEDIA_MODIFIED_TIME,
		.func = _compare_cb_date,
	},
	[E_SORT_NAME] = {
		.order_type = MEDIA_CONTENT_ORDER_ASC,
		.order_keyword = MEDIA_TITLE,
		.func = _compare_cb_name,
	}
};

static struct _list_info g_list_info[E_LIST_MAX] = {
	[E_LIST_NAME] = {
		.media_cmp = _compare_title,
		.name_get = _get_title,
		.data_get = _get_data_title,
	},
	[E_LIST_DATE] = {
		.media_cmp = _compare_time,
		.name_get = _get_time,
		.data_get = _get_data_time,
	}
};

static bool _create_filter(struct _data *data, filter_h *filter, int type)
{
	filter_h tmp_filter;
	int ret;
	char buf[1024];

	ret = media_filter_create(&tmp_filter);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to create media filter");
		return false;
	}

	snprintf(buf, sizeof(buf), "%s", data->media_type);

	if (type == E_FILTER_FOLDER && data->source_type != E_SOURCE_ALL) {
		char s[64];

		snprintf(s, sizeof(s), " AND MEDIA_STORAGE_TYPE=%d",
					data->source_type);
		strcat(buf, s);
	}

	media_filter_set_condition(tmp_filter, buf,
			MEDIA_CONTENT_COLLATE_DEFAULT);

	media_filter_set_order(tmp_filter,
			g_sort_info[data->sort_type].order_type,
			g_sort_info[data->sort_type].order_keyword,
			MEDIA_CONTENT_COLLATE_NOCASE);

	*filter = tmp_filter;

	return true;
}

static char *_get_date_string(struct tm *tm)
{
	char buf[32];

	strftime(buf, sizeof(buf), "%Y.%m.%d", tm);

	return strdup(buf);
}

static int _compare_title(struct group_info *gi, app_media_info *info)
{
	if (!gi || !gi->data || !info->title)
		return -1;

	return strncasecmp(gi->data, info->title, 1);
}

static void *_get_data_title(app_media_info *info)
{
	if (!info->title)
		return NULL;

	return strdup(info->title);
}

static char *_get_title(app_media_info *info)
{
	if (!info->title)
		return NULL;

	return strndup(info->title, 1);
}

static int _compare_time(struct group_info *gi, app_media_info *info)
{
	struct tm *tm;

	if (!gi || !gi->data)
		return -1;

	tm = gi->data;

	if (tm->tm_year == info->content_time->tm_year &&
			tm->tm_yday == info->content_time->tm_yday)
		return 0;

	return 1;
}

static void *_get_data_time(app_media_info *info)
{
	struct tm *tm;

	tm = calloc(1, sizeof(*tm));
	if (!tm)
		return NULL;

	memcpy(tm, info->content_time, sizeof(*tm));

	return tm;
}

static char *_get_time(app_media_info *info)
{
	return  _get_date_string(info->content_time);
}

static int _compare_cb_date(const void *data1, const void *data2)
{
	app_media *am1, *am2;
	app_media_info *info1, *info2;
	time_t tm1, tm2;

	am1 = (app_media *)data1;
	am2 = (app_media *)data2;

	info1 = app_media_get_info(am1);
	info2 = app_media_get_info(am2);

	if (!info1 || !info2)
		return -1;

	tm1 = mktime(info1->content_time);
	tm2 = mktime(info2->content_time);

	if (tm1 < tm2)
		return 1;

	return -1;
}

static int _compare_cb_name(const void *data1, const void *data2)
{
	app_media *am1, *am2;
	app_media_info *info1, *info2;

	am1 = (app_media *)data1;
	am2 = (app_media *)data2;

	info1 = app_media_get_info(am1);
	info2 = app_media_get_info(am2);

	if (!info1 || !info2 || !info1->display_name || !info2->display_name)
		return -1;

	return strcasecmp(info1->display_name, info2->display_name);
}

static Eina_List *_sort_list(Eina_List *list, int sort)
{
	Eina_List *sorted_list;

	sorted_list = eina_list_sort(list, 0, g_sort_info[sort].func);

	return sorted_list;
}

static bool _get_each_media_info(media_info_h media_h, void *dt)
{
	app_media *am;
	struct _data *data;

	if (!dt)
		return false;

	data = dt;

	am = app_media_create(media_h);
	if (!am) {
		_ERR("failed to create app media");
		return false;
	}

	data->media_list = eina_list_append(data->media_list, am);

	return true;
}

static bool _get_medialist(struct _data *data)
{
	filter_h filter;
	int ret;

	if (!_create_filter(data, &filter, E_FILTER_MEDIA)) {
		_ERR("failed to create filter");
		return false;
	}

	ret = media_info_foreach_media_from_db(filter,
			_get_each_media_info, data);
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to get folder info");
		media_filter_destroy(filter);
		return false;
	}

	media_filter_destroy(filter);

	return true;
}

struct mediadata *mediadata_create(const char *media_type,
			int source_type, int sort_type)
{
	struct mediadata *md;
	struct _data data;
	int ret;

	if (!media_type || source_type < 0 || source_type >= E_SOURCE_MAX
		|| sort_type < 0 || sort_type >= E_SORT_MAX) {
		_ERR("invalid argument");
		return NULL;
	}

	md = calloc(1, sizeof(*md));
	if (!md) {
		_ERR("failed to create mediadata");
		return NULL;
	}

	ret = media_content_connect();
	if (ret != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to connect to media content");
		free(md);
		return NULL;
	}

	data.media_type = media_type;
	data.source_type = source_type;
	data.sort_type = sort_type;
	data.media_list = NULL;

	if (!_get_medialist(&data)) {
		_ERR("failed to get folderlist");
		free(md);
		media_content_disconnect();
		return NULL;
	}

	media_content_disconnect();

	md->media_list = _sort_list(data.media_list, data.sort_type);

	return md;
}

void mediadata_destroy(struct mediadata *md)
{
	if (!md) {
		_ERR("failed to get mediadata");
		return;
	}

	eina_list_free(md->media_list);
}

Eina_List *mediadata_get_medialist(struct mediadata *md)
{
	if (!md) {
		_ERR("failed to get mediadata");
		return NULL;
	}

	return md->media_list;
}

Eina_List *mediadata_get_list(struct mediadata *md, int list_type)
{
	Eina_List *list, *l;
	struct group_info *gi;
	app_media *am;
	app_media_info *info;

	if (!md) {
		_ERR("failed to get mediadata");
		return NULL;
	}

	if (list_type < 0 || list_type >= E_LIST_MAX) {
		_ERR("invalid argument");
		return NULL;
	}

	gi = NULL;
	list = NULL;
	EINA_LIST_FOREACH(md->media_list, l, am) {
		info = app_media_get_info(am);
		if (!info) {
			_ERR("failed to get media info");
			mediadata_free_list(list);
			return NULL;
		}

		if (g_list_info[list_type].media_cmp(gi, info)) {
			gi = calloc(1, sizeof(*gi));
			if (!gi) {
				_ERR("failed to create group info");
				mediadata_free_list(list);
				return NULL;
			}

			gi->name = g_list_info[list_type].name_get(info);
			gi->data = g_list_info[list_type].data_get(info);

			list = eina_list_append(list, gi);
		}

		gi->list = eina_list_append(gi->list, am);

		md->media_count[info->media_type]++;
	}

	return list;
}

int mediadata_get_media_count(struct mediadata *md, int media_type)
{
	if (!md) {
		_ERR("failed to get mediadata");
		return -1;
	}

	if (media_type < 0 || media_type >= E_MEDIA_MAX) {
		_ERR("invalid argument");
		return -1;
	}

	return md->media_count[media_type];
}

void mediadata_free_list(Eina_List *list)
{
	Eina_List *l;
	struct group_info *gi;

	EINA_LIST_FOREACH(list, l, gi) {
		free(gi->name);
		free(gi->data);
		eina_list_free(gi->list);

		free(gi);
	}
}
