/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdbool.h>
#include <Elementary.h>
#include <app_debug.h>
#include <inputmgr.h>

#include "define.h"
#include "util/controller.h"

struct _event_cb {
	void (*func)(void *data, const char *ev);
	void *data;
};

struct _control {
	Evas_Object *obj;
	char *part;
};

struct _priv {
	Evas_Object *base;
	Eina_List *list;
	struct _event_cb cb;
};

static void _btn_cb(void *data, Evas_Object *obj,
		const char *emission, const char *source)
{
	struct _priv *p;

	if (!data) {
		_ERR("invalid parameter");
		return;
	}

	p = data;

	if (p->cb.func)
		p->cb.func(p->cb.data, source);
}

static void _mouse_move_cb(int id, void *data, Evas *e,
			Evas_Object *obj, Evas_Event_Mouse_Move *ev)
{
	if (!elm_object_focus_get(obj))
		elm_object_focus_set(obj, EINA_TRUE);
}

static void _clicked_cb(int id, void *data, Evas_Object *obj)
{
	elm_object_signal_emit(obj, SIG_BTN_CLICKED, "");
}

static input_handler _handler = {
	.mouse_move = _mouse_move_cb,
	.clicked = _clicked_cb,
};

static Evas_Object *_add_btn(Evas_Object *base, const char *style)
{
	Evas_Object *btn;

	btn = elm_button_add(base);
	if (!btn)
		return NULL;

	elm_object_style_set(btn, style);

	return btn;
}

static bool _add_control(void *handle, const char *name, int loc,
		const char *style, const char *part)
{
	struct _priv *p;
	struct _control *ctl;
	Evas_Object *btn;
	char buf[32];

	if (!handle || !name || !style || !part) {
		_ERR("invalid parameter");
		return false;
	}

	p = handle;

	snprintf(buf, sizeof(buf), "%s_%s", style, name);
	btn = _add_btn(p->base, buf);
	if (!btn) {
		_ERR("failed to add button");
		return false;
	}

	snprintf(buf, sizeof(buf), "%s%d", part, loc);

	ctl = calloc(1, sizeof(*ctl));
	if (!ctl) {
		_ERR("failed to allocate");
		return false;
	}

	ctl->obj = btn;
	ctl->part = strdup(buf);

	p->list = eina_list_append(p->list, ctl);

	inputmgr_add_callback(btn, 0, &_handler, p);
	elm_object_signal_callback_add(btn, SIG_BTN_CALLBACK, name, _btn_cb, p);

	return true;
}

static void _add_callback(void *handle,
		void (*func)(void *, const char *), void *data)
{
	struct _priv *p;

	if (!handle) {
		_ERR("invalid parameter");
		return;
	}

	p = handle;
	p->cb.func = func;
	p->cb.data = data;
}

static void _focus(void *handle, int loc, bool foc)
{
	struct _priv *p;
	struct _control *ctl;

	if (!handle) {
		_ERR("invalid parameter");
		return;
	}

	p = handle;
	ctl = eina_list_nth(p->list, loc);

	if (ctl && ctl->obj)
		elm_object_focus_set(ctl->obj, foc);
}

static void _signal(void *handle, int loc, const char *signal)
{
	struct _priv *p;
	struct _control *ctl;

	if (!handle) {
		_ERR("invalid parameter");
		return;
	}

	p = handle;
	ctl = eina_list_nth(p->list, loc);

	if (ctl && ctl->obj)
		elm_object_signal_emit(ctl->obj, signal, "");
}

static void _enable(void *handle)
{
	struct _priv *p;
	struct _control *ctl;
	Eina_List *l;

	if (!handle) {
		_ERR("invalid parameter");
		return;
	}

	p = handle;

	EINA_LIST_FOREACH(p->list, l, ctl) {
		if (!ctl->obj)
			continue;

		evas_object_freeze_events_set(ctl->obj, EINA_FALSE);
		elm_object_focus_allow_set(ctl->obj, EINA_TRUE);
	}
}

static void _disable(void *handle)
{
	struct _priv *p;
	struct _control *ctl;
	Eina_List *l;

	if (!handle) {
		_ERR("invalid parameter");
		return;
	}

	p = handle;

	EINA_LIST_FOREACH(p->list, l, ctl) {
		if (!ctl->obj)
			continue;

		evas_object_freeze_events_set(ctl->obj, EINA_TRUE);
		elm_object_focus_allow_set(ctl->obj, EINA_FALSE);
	}
}

static void _show(void *handle)
{
	struct _priv *p;
	struct _control *ctl;
	Eina_List *l;

	if (!handle) {
		_ERR("invalid parameter");
		return;
	}

	p = handle;

	EINA_LIST_FOREACH(p->list, l, ctl) {
		if (!ctl->obj)
			continue;

		evas_object_show(ctl->obj);
		elm_object_part_content_set(p->base, ctl->part, ctl->obj);
	}

	_enable(handle);
}

static void _hide(void *handle)
{
	struct _priv *p;
	struct _control *ctl;
	Eina_List *l;

	if (!handle) {
		_ERR("invalid parameter");
		return;
	}

	p = handle;

	EINA_LIST_FOREACH(p->list, l, ctl) {
		if (!ctl->obj)
			continue;

		elm_object_focus_set(ctl->obj, false);

		evas_object_hide(ctl->obj);
		elm_object_part_content_unset(p->base, ctl->part);
	}

	_disable(handle);
}

static Evas_Object *_get_object(void *handle, int loc)
{
	struct _priv *p;
	struct _control *ctl;

	if (!handle) {
		_ERR("invalid parameter");
		return NULL;
	}

	p = handle;

	ctl = eina_list_nth(p->list, loc);
	if (!ctl)
		return NULL;

	return ctl->obj;
}

static int _get_count(void *handle)
{
	struct _priv *p;

	if (!handle) {
		_ERR("invalid parameter");
		return 0;
	}

	p = handle;

	return eina_list_count(p->list);
}

static struct controller_ops _operations = {
	.add_control = _add_control,
	.add_callback = _add_callback,
	.focus = _focus,
	.signal = _signal,
	.enable = _enable,
	.disable = _disable,
	.show = _show,
	.hide = _hide,
	.get_object = _get_object,
	.get_count = _get_count,
};

struct controller *controller_create(Evas_Object *base)
{
	struct controller *m;
	struct _priv *p;

	if (!base) {
		_ERR("invalid parameter");
		return NULL;
	}

	m = calloc(1, sizeof(*m));
	if (!m) {
		_ERR("failed to allocate");
		return NULL;
	}

	p = calloc(1, sizeof(*p));
	if (!p) {
		_ERR("failed to allocate");
		free(m);
		return NULL;
	}

	p->base = base;

	m->handle = (void *)p;
	m->ops = &_operations;

	return m;
}

void controller_destroy(struct controller *m)
{
	struct _priv *p;
	struct _control *ctl;

	if (!m)
		return;

	p = m->handle;
	if (!p)
		goto out;

	EINA_LIST_FREE(p->list, ctl) {
		if (ctl->obj)
			inputmgr_remove_callback(ctl->obj, &_handler);
		free(ctl->part);
		free(ctl);
	}

	free(p);

out:
	free(m);
}
