/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <app.h>
#include <Elementary.h>
#include <app_debug.h>
#include <inputmgr.h>
#include <viewmgr.h>
#include <layoutmgr.h>

#include "define.h"
#include "layout.h"
#include "view.h"

#define TITLE_TEXT "Media Hub"

#define MENU_BTN_SIZE 3
#define BOX_PADDING_SIZE 80

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *btn[MENU_BTN_SIZE];
	Evas_Object *focused_btn;

	layoutmgr *lmgr;

	int current_layout;
};

struct _menu_item {
	const char *name;
	const char *layout_id;
	layout_class *(*func_get_lclass)(void);
};

static struct _menu_item g_menu_item[MENU_BTN_SIZE] = {
	{
		"Movie",
		LAYOUT_MOVIE,
		layout_movie_get_lclass
	},
	{
		"Gallery",
		LAYOUT_GALLERY,
		layout_gallery_get_lclass
	},
	{
		"Music",
		LAYOUT_MUSIC,
		layout_music_get_lclass
	}
};

static void _key_down_cb(int id, void *data, Evas *e, Evas_Object *obj,
			Evas_Event_Key_Down *ev)
{
	if (!obj || !ev)
		return;

	if (!strcmp(ev->keyname, KEY_ENTER) ||
		!strcmp(ev->keyname, KEY_ENTER_REMOTE))
		elm_object_focus_next(obj, ELM_FOCUS_DOWN);
	else if (!strcmp(ev->keyname, KEY_BACK) ||
		!strcmp(ev->keyname, KEY_BACK_REMOTE))
		ui_app_exit();
}

static void _mouse_move_cb(int id, void *data, Evas *e,
			Evas_Object *obj, Evas_Event_Mouse_Move *ev)
{
	if (!elm_object_focus_get(obj))
		elm_object_focus_set(obj, EINA_TRUE);
}

static void _focused_cb(int id, void *data, Evas_Object *obj,
			Elm_Object_Item *it)
{
	struct _priv *priv;
	int i;

	if (!data) {
		_ERR("failed to get data");
		return;
	}

	priv = data;

	if (priv->focused_btn == obj)
		return;

	if (priv->focused_btn)
		elm_object_signal_emit(priv->focused_btn, SIG_BTN_UNSELECTED,
					SIG_SOURCE_SRC);

	for (i = 0; i < MENU_BTN_SIZE; i++) {
		if (priv->btn[i] == obj)
			break;
	}

	if (i == MENU_BTN_SIZE)
		return;

	layoutmgr_hide_layout(priv->lmgr,
			g_menu_item[priv->current_layout].layout_id);
	layoutmgr_show_layout(priv->lmgr, g_menu_item[i].layout_id);
	layoutmgr_update_layout(priv->lmgr, g_menu_item[i].layout_id,
			UPDATE_CONTENT, NULL);

	priv->focused_btn = obj;
	priv->current_layout = i;
}

static input_handler handler = {
	.key_down = _key_down_cb,
	.mouse_move = _mouse_move_cb,
	.focused = _focused_cb
};

static bool _draw_title(struct _priv *priv)
{
	if (!priv)
		return false;

	elm_object_part_text_set(priv->base, PART_TITLE, TITLE_TEXT);

	return true;
}

static bool _draw_menu_btn(struct _priv *priv)
{
	Evas_Object *box, *btn;
	int i;

	if (!priv)
		return false;

	box = elm_box_add(priv->base);
	if (!box) {
		_ERR("failed to create box object");
		return false;
	}

	elm_box_horizontal_set(box, EINA_TRUE);
	elm_box_padding_set(box, BOX_PADDING_SIZE, 0);

	evas_object_size_hint_weight_set(box,
			EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);

	for (i = 0; i < MENU_BTN_SIZE; i++) {
		btn = elm_button_add(box);
		if (!btn) {
			_ERR("failed to create button object");
			return false;
		}

		elm_object_style_set(btn, STYLE_BTN_MENU);
		elm_object_text_set(btn, g_menu_item[i].name);

		elm_box_pack_end(box, btn);

		evas_object_show(btn);

		inputmgr_add_callback(btn, 0, &handler, priv);

		priv->btn[i] = btn;
	}

	evas_object_show(box);

	elm_object_part_content_set(priv->base, PART_MENU_AREA, box);

	elm_object_focus_next_object_set(priv->btn[MENU_BTN_SIZE - 1],
				priv->btn[0], ELM_FOCUS_RIGHT);
	elm_object_focus_next_object_set(priv->btn[0],
				priv->btn[MENU_BTN_SIZE - 1], ELM_FOCUS_LEFT);

	return true;
}

static bool _draw_items(struct _priv *priv)
{
	if (!priv)
		return false;

	if (!_draw_title(priv))
		return false;

	if (!_draw_menu_btn(priv))
		return false;

	return true;
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	Evas_Object *base;
	layoutmgr *lmgr;
	int i;

	if (!win) {
		_ERR("failed to get win object");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to allocate priv");
		return NULL;
	}

	base = elm_layout_add(win);
	if (!base) {
		_ERR("failed to create base object");
		free(priv);
		return NULL;
	}

	elm_layout_file_set(base, EDJEFILE, GRP_BASE_VIEW);

	evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND,
				EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, base);

	priv->win = win;
	priv->base = base;

	if (!_draw_items(priv)) {
		_ERR("failed to draw items");
		free(priv);
		return NULL;
	}

	lmgr = layoutmgr_create(base);
	for (i = 0; i < MENU_BTN_SIZE; i++)
		layoutmgr_add_layout(lmgr, g_menu_item[i].func_get_lclass(),
					priv->btn[i]);

	priv->lmgr = lmgr;

	viewmgr_set_view_data(VIEW_BASE, priv);

	elm_object_focus_set(priv->btn[0], EINA_TRUE);

	return base;
}

static void _show(void *view_data)
{
	struct _priv *priv;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	evas_object_show(priv->base);
}

static void _hide(void *view_data)
{
	struct _priv *priv;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	evas_object_hide(priv->base);
}

static void _update(void *view_data, int update_type, void *data)
{
	struct _priv *priv;
	struct view_update_data *vdata;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	if (!data) {
		_ERR("invalid argument");
		return;
	}

	priv = view_data;
	vdata = data;

	layoutmgr_update_layout(priv->lmgr,
			g_menu_item[priv->current_layout].layout_id,
			update_type, vdata);
}

static void _pause(void *view_data)
{
	viewmgr_update_view(VIEW_MPLAYER, UPDATE_PLAYER, NULL);
}

static void _resume(void *view_data)
{
	struct _priv *priv;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	layoutmgr_update_layout(priv->lmgr,
			g_menu_item[priv->current_layout].layout_id,
			UPDATE_RESUME, NULL);
}

static void _destroy(void *view_data)
{
	struct _priv *priv;
	int i;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	for (i = 0; i < MENU_BTN_SIZE; i++)
		inputmgr_remove_callback(priv->btn[i], &handler);

	layoutmgr_remove_layout(priv->lmgr, LAYOUT_MOVIE);
	layoutmgr_remove_layout(priv->lmgr, LAYOUT_GALLERY);
	layoutmgr_remove_layout(priv->lmgr, LAYOUT_MUSIC);

	layoutmgr_destroy(priv->lmgr);

	evas_object_del(priv->base);

	free(priv);
}

static view_class _vclass = {
	.view_id = VIEW_BASE,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.update = _update,
	.pause = _pause,
	.resume = _resume,
	.destroy = _destroy,
};

view_class *view_base_get_vclass(void)
{
	return &_vclass;
}
